@@ -10,7 +10,8 @@ class Object:
     self.world = world
     self.pos = np.array(pos)
     self.random = world.random
-    self.health = 0
+    self.inventory = {'health': 0}
+    self.removed = False
 
   @property
   def texture(self):
@@ -20,6 +21,18 @@ class Object:
   def walkable(self):
     return constants.walkable
 
+  @property
+  def health(self):
+    return self.inventory['health']
+
+  @health.setter
+  def health(self, value):
+    self.inventory['health'] = value
+
+  @property
+  def all_dirs(self):
+    return ((-1, 0), (+1, 0), (0, -1), (0, +1))
+
   def move(self, direction):
     direction = np.array(direction)
     target = self.pos + direction
@@ -49,30 +62,29 @@ class Object:
       return np.array((0, np.sign(offset[1])))
 
   def random_dir(self):
-    directions = ((-1, 0), (+1, 0), (0, -1), (0, +1))
-    return directions[self.random.randint(0, len(directions))]
+    return self.all_dirs[self.random.randint(0, 4)]
 
 
 class Player(Object):
 
-  def __init__(self, world, pos, health):
+  def __init__(self, world, pos):
     super().__init__(world, pos)
     self.facing = (0, 1)
-    self.health = health
-    self.inventory = {item: 0 for item in constants.items}
+    self.inventory = {
+        name: info['initial'] for name, info in constants.items.items()}
     self.achievements = {name: 0 for name in constants.achievements}
-    self._max_health = health
+    self.action = 'noop'
+    self.sleeping = False
+    self._last_health = self.health
     self._hunger = 0
-
-
-    # TODO
-    self.inventory['wood'] = 4
-    self.inventory['stone'] = 3
-    self.inventory['wood_pickaxe'] = 2
-    self.inventory['stone_pickaxe'] = 1
+    self._thirst = 0
+    self._fatigue = 0
+    self._recover = 0
 
   @property
   def texture(self):
+    if self.sleeping:
+      return 'player-sleep'
     return {
         (-1, 0): 'player-left',
         (+1, 0): 'player-right',
@@ -84,28 +96,79 @@ class Player(Object):
   def walkable(self):
     return constants.walkable + ['lava']
 
-  def update(self, action):
-    self._hunger += 1
-    if self._hunger > 100:
-      self.health -= 1
-      self._hunger = 0
+  def update(self):
     target = (self.pos[0] + self.facing[0], self.pos[1] + self.facing[1])
     material, obj = self.world[target]
-    action = constants.actions[action]
+    action = self.action
+    if self.sleeping:
+      if self.inventory['energy'] < constants.items['energy']['max']:
+        action = 'sleep'
+      else:
+        self.sleeping = False
     if action == 'noop':
       pass
     elif action.startswith('move_'):
       self._move(action[len('move_'):])
     elif action == 'do' and obj:
-      self._interact(obj)
+      self._do_object(obj)
     elif action == 'do':
-      self._collect(target, material)
+      self._do_material(target, material)
+    elif action == 'sleep':
+      if self.inventory['energy'] < constants.items['energy']['max']:
+        self.sleeping = True
     elif action.startswith('place_'):
       self._place(action[len('place_'):], target, material)
     elif action.startswith('make_'):
       self._make(action[len('make_'):])
-    for item, amount in self.inventory.items():
-      self.inventory[item] = max(0, min(amount, 5))
+    self._update_life_stats()
+    self._degen_or_regen_health()
+    for name, amount in self.inventory.items():
+      maxmium = constants.items[name]['max']
+      self.inventory[name] = max(0, min(amount, maxmium))
+    # This needs to happen after the inventory states are clamped
+    # because it involves the health water inventory count.
+    self._wake_up_when_hurt()
+
+  def _update_life_stats(self):
+    self._hunger += 0.5 if self.sleeping else 1
+    if self._hunger > 25:
+      self._hunger = 0
+      self.inventory['food'] -= 1
+    self._thirst += 0.5 if self.sleeping else 1
+    if self._thirst > 20:
+      self._thirst = 0
+      self.inventory['drink'] -= 1
+    if self.sleeping:
+      self._fatigue = min(self._fatigue - 1, 0)
+    else:
+      self._fatigue += 1
+    if self._fatigue < -10:
+      self._fatigue = 0
+      self.inventory['energy'] += 1
+    if self._fatigue > 30:
+      self._fatigue = 0
+      self.inventory['energy'] -= 1
+
+  def _degen_or_regen_health(self):
+    necessities = (
+        self.inventory['food'] > 0,
+        self.inventory['drink'] > 0,
+        self.inventory['energy'] > 0 or self.sleeping)
+    if all(necessities):
+      self._recover += 2 if self.sleeping else 1
+    else:
+      self._recover -= 0.5 if self.sleeping else 1
+    if self._recover > 25:
+      self._recover = 0
+      self.health += 1
+    if self._recover < -15:
+      self._recover = 0
+      self.health -= 1
+
+  def _wake_up_when_hurt(self):
+    if self.health < self._last_health:
+      self.sleeping = False
+    self._last_health = self.health
 
   def _move(self, direction):
     directions = dict(left=(-1, 0), right=(+1, 0), up=(0, -1), down=(0, +1))
@@ -114,13 +177,22 @@ class Player(Object):
     if self.world[self.pos][0] == 'lava':
       self.health = 0
 
-  def _interact(self, obj):
+  def _do_object(self, obj):
     damage = max([
         1,
         self.inventory['wood_sword'] and 2,
         self.inventory['stone_sword'] and 3,
         self.inventory['iron_sword'] and 5,
     ])
+    if isinstance(obj, Plant):
+      if obj.ripe:
+        obj.grown = 0
+        self.inventory['food'] += 4
+        self.achievements['eat_plant'] += 1
+    if isinstance(obj, Fence):
+      self.world.remove(obj)
+      self.inventory['fence'] += 1
+      self.achievements['collect_fence'] += 1
     if isinstance(obj, Zombie):
       obj.health -= damage
       if obj.health <= 0:
@@ -132,11 +204,17 @@ class Player(Object):
     if isinstance(obj, Cow):
       obj.health -= damage
       if obj.health <= 0:
-        self.health = min(self.health + 1, self._max_health)
+        self.inventory['food'] += 6
+        self.achievements['eat_cow'] += 1
+        # TODO: Keep track of previous inventory state to do this in a more
+        # general way.
         self._hunger = 0
-        self.achievements['find_food'] += 1
 
-  def _collect(self, target, material):
+  def _do_material(self, target, material):
+    if material == 'water':
+      # TODO: Keep track of previous inventory state to do this in a more
+      # general way.
+      self._thirst = 0
     info = constants.collect.get(material)
     if not info:
       return
@@ -144,11 +222,14 @@ class Player(Object):
       if self.inventory[name] < amount:
         return
     self.world[target] = info['leaves']
-    for name, amount in info['receive'].items():
-      self.inventory[name] += 1
-    self.achievements[f'collect_{material}'] += 1
+    if self.random.uniform() <= info.get('probability', 1):
+      for name, amount in info['receive'].items():
+        self.inventory[name] += amount
+        self.achievements[f'collect_{name}'] += 1
 
   def _place(self, name, target, material):
+    if self.world[target][1]:
+      return
     info = constants.place[name]
     if material not in info['where']:
       return
@@ -156,11 +237,18 @@ class Player(Object):
       return
     for item, amount in info['uses'].items():
       self.inventory[item] -= amount
-    self.world[target] = name
+    if info['type'] == 'material':
+      self.world[target] = name
+    elif info['type'] == 'object':
+      cls = {
+          'fence': Fence,
+          'plant': Plant,
+      }[name]
+      self.world.add(cls(self.world, target))
     self.achievements[f'place_{name}'] += 1
 
   def _make(self, name):
-    nearby = self.world.nearby(self.pos, 2)
+    nearby, _ = self.world.nearby(self.pos, 1)
     info = constants.make[name]
     if not all(util in nearby for util in info['nearby']):
       return
@@ -168,7 +256,7 @@ class Player(Object):
       return
     for item, amount in info['uses'].items():
       self.inventory[item] -= amount
-    self.inventory[name] += 1
+    self.inventory[name] += info['gives']
     self.achievements[f'make_{name}'] += 1
 
 
@@ -186,7 +274,8 @@ class Cow(Object):
     if self.health <= 0:
       self.world.remove(self)
     if self.random.uniform() < 0.5:
-      self.move(self.random_dir())
+      direction = self.random_dir()
+      self.move(direction)
 
 
 class Zombie(Object):
@@ -195,7 +284,7 @@ class Zombie(Object):
     super().__init__(world, pos)
     self.player = player
     self.health = 5
-    self.near = False
+    self.cooldown = 0
 
   @property
   def texture(self):
@@ -205,17 +294,21 @@ class Zombie(Object):
     if self.health <= 0:
       self.world.remove(self)
     dist = self.distance(self.player)
-    if dist <= 1:
-      if not self.near:
-        self.near = True
-      elif self.random.uniform() > 0.7:
-        self.player.health -= 1
-    else:
-      self.near = False
-    if dist <= 6 and self.random.uniform() < 0.8:
-      self.move(self.toward(self.player, self.random.uniform() < 0.7))
+    if dist <= 8 and self.random.uniform() < 0.9:
+      self.move(self.toward(self.player, self.random.uniform() < 0.8))
     else:
       self.move(self.random_dir())
+    dist = self.distance(self.player)
+    if dist <= 1:
+      if self.cooldown:
+        self.cooldown -= 1
+      else:
+        if self.player.sleeping:
+          damage = max(2, self.player.health - 2)
+        else:
+          damage = 2
+        self.player.health -= damage
+        self.cooldown = 5
 
 
 class Skeleton(Object):
@@ -280,9 +373,51 @@ class Arrow(Object):
     target = self.pos + self.facing
     material, obj = self.world[target]
     if obj:
-      obj.health -= 1
+      obj.health -= 2
       self.world.remove(self)
     elif material not in self.walkable:
       self.world.remove(self)
+      if material in ['table', 'furnace']:
+        self.world[target] = 'path'
     else:
       self.move(self.facing)
+
+
+class Plant(Object):
+
+  def __init__(self, world, pos):
+    super().__init__(world, pos)
+    self.health = 1
+    self.grown = 0
+
+  @property
+  def texture(self):
+    if self.ripe:
+      return 'plant-ripe'
+    else:
+      return 'plant'
+
+  @property
+  def ripe(self):
+    return self.grown > 300
+
+  def update(self):
+    self.grown += 1
+    objs = [self.world[self.pos + dir_][1] for dir_ in self.all_dirs]
+    if any(isinstance(obj, (Zombie, Skeleton, Cow)) for obj in objs):
+      self.health -= 1
+    if self.health <= 0:
+      self.world.remove(self)
+
+
+class Fence(Object):
+
+  def __init__(self, world, pos):
+    super().__init__(world, pos)
+
+  @property
+  def texture(self):
+    return 'fence'
+
+  def update(self):
+    pass
