@@ -14,13 +14,14 @@ def main():
   parser.add_argument('--view', type=int, nargs=2, default=(9, 9))
   parser.add_argument('--size', type=int, nargs=2, default=(64, 64))
   parser.add_argument('--length', type=int, default=10000)
-  parser.add_argument('--health', type=int, default=5)
+  parser.add_argument('--health', type=int, default=9)
   parser.add_argument('--record', type=str, default=None)
   args = parser.parse_args()
 
   random = np.random.RandomState(args.seed)
-  env = crafter.Env(
-      args.area, args.view, args.size, args.length, args.health, args.seed)
+  crafter.constants.items['health']['max'] = args.health
+  crafter.constants.items['health']['initial'] = args.health
+  env = crafter.Env(args.area, args.view, args.size, args.length, args.seed)
   if args.record:
     frames = []
 
@@ -34,7 +35,7 @@ def main():
     action = random.randint(0, env.action_space.n)
     obs, _, done, _ = env.step(action)
     if args.record:
-      frames.append(obs['image'])
+      frames.append(obs)
   duration = time.time() - start
   step = env._step
   print(f'Step time: {1000*duration/step:.2f}ms ({int(step/duration)} FPS)')
